@@ -14,9 +14,9 @@ import traceback
 
 from gi.repository import Gio
 
-from Classes import Update, Alias, Rule
+from Classes import Update, Alias, Rule, KERNEL_PKG_NAMES, META_NAMES
 
-gettext.install("pearlupdate", "/usr/share/pearllinux/locale")
+gettext.install("pearlupdate", "/usr/share/locale")
 
 class KernelVersion():
 
@@ -82,17 +82,12 @@ class APTCheck():
 
     def refresh_cache(self):
         if os.getuid() == 0 :
-            use_synaptic = False
-            if (len(sys.argv) > 1):
-                if sys.argv[1] == "--use-synaptic":
-                    use_synaptic = True
-
-            if use_synaptic:
+            if "--use-synaptic" in sys.argv:
                 window_id = int(sys.argv[2])
-                from subprocess import Popen, PIPE
+                from subprocess import Popen
                 cmd = ["sudo", "/usr/sbin/synaptic", "--hide-main-window", "--update-at-startup", "--non-interactive", "--parent-window-id", "%d" % window_id]
                 comnd = Popen(' '.join(cmd), shell=True)
-                returnCode = comnd.wait()
+                comnd.wait()
             else:
                 self.cache.update()
 
@@ -112,63 +107,66 @@ class APTCheck():
         # Kernel updates
         try:
             if self.settings.get_boolean("kernel-updates-are-visible"):
+
                 # Get the uname version
                 uname_kernel = KernelVersion(platform.release())
 
-                # Get the recommended version
-                if 'linux-image-generic' in self.cache:
-                    recommended_kernel = KernelVersion(self.cache['linux-image-generic'].candidate.version)
-                    if (uname_kernel.numeric_representation <= recommended_kernel.numeric_representation):
-                        for pkgname in ['linux-headers-VERSION', 'linux-headers-VERSION-generic', 'linux-image-VERSION-generic', 'linux-image-extra-VERSION-generic']:
-                            pkgname = pkgname.replace('VERSION', recommended_kernel.std_version)
-                            if pkgname in self.cache:
-                                pkg = self.cache[pkgname]
-                                if not pkg.is_installed:
-                                    self.add_update(pkg, kernel_update=True)
-                        return
+                # Check if any meta is installed..
+                meta_installed = False
+                for meta_name in META_NAMES:
+                    if meta_name in self.cache:
+                        meta = self.cache[meta_name]
+                        if meta.is_installed:
+                            meta_installed = True
+                            break
 
-                # We're using a series which is more recent than the recommended one, check the HWE kernel first
-                if 'linux-image-generic-hwe-16.04' in self.cache:
-                    recommended_kernel = KernelVersion(self.cache['linux-image-generic-hwe-16.04'].candidate.version)
-                    if (uname_kernel.numeric_representation <= recommended_kernel.numeric_representation):
-                        for pkgname in ['linux-headers-VERSION', 'linux-headers-VERSION-generic', 'linux-image-VERSION-generic', 'linux-image-extra-VERSION-generic']:
-                            pkgname = pkgname.replace('VERSION', recommended_kernel.std_version)
-                            if pkgname in self.cache:
-                                pkg = self.cache[pkgname]
-                                if not pkg.is_installed:
-                                    self.add_update(pkg, kernel_update=True)
-                        return
+                # If no meta is installed, try to recommend one
+                if not meta_installed:
+                    for meta_name in META_NAMES:
+                        if meta_name in self.cache:
+                            meta = self.cache[meta_name]
+                            recommended_kernel = KernelVersion(meta.candidate.version)
+                            if (uname_kernel.numeric_representation <= recommended_kernel.numeric_representation):
+                                # This meta version is >= to the uname version, add it as an update
+                                self.add_update(meta, kernel_update=True)
+                                # Return because we only want to add one meta, and we don't want to recommend latest kernels in the series
+                                return
 
                 # We've gone past all the metas, so we should recommend the latest kernel on the series we're in
                 max_kernel = uname_kernel
+                kernel_type = "-generic"
+                if self.settings.get_boolean("use-lowlatency-kernels"):
+                    kernel_type = "-lowlatency"
                 for pkg in self.cache:
                     package_name = pkg.name
-                    if (package_name.startswith("linux-image-3") or package_name.startswith("linux-image-4")) and package_name.endswith("-generic"):
-                        version = package_name.replace("linux-image-", "").replace("-generic", "")
+                    if (package_name.startswith("linux-image-3") or package_name.startswith("linux-image-4")) and package_name.endswith(kernel_type):
+                        version = package_name.replace("linux-image-", "").replace("-generic", "").replace("-lowlatency", "")
                         kernel = KernelVersion(version)
                         if kernel.numeric_representation > max_kernel.numeric_representation and kernel.series == max_kernel.series:
                             max_kernel = kernel
                 if max_kernel.numeric_representation != uname_kernel.numeric_representation:
-                    for pkgname in ['linux-headers-VERSION', 'linux-headers-VERSION-generic', 'linux-image-VERSION-generic', 'linux-image-extra-VERSION-generic']:
+                    for pkgname in KERNEL_PKG_NAMES:
                         pkgname = pkgname.replace('VERSION', max_kernel.std_version)
                         if pkgname in self.cache:
                             pkg = self.cache[pkgname]
                             if not pkg.is_installed:
                                 self.add_update(pkg, kernel_update=True)
 
-        except Exception as e:
+        except Exception:
             print(sys.exc_info()[0])
 
     def add_update(self, package, kernel_update=False):
 
         if package.name in ['linux-libc-dev', 'linux-kernel-generic']:
             source_name = package.name
-        elif package.name.startswith("linux-image") or package.name.startswith("linux-headers"):
-            source_name = package.name.replace("-generic", "").replace("-extra", "").replace("-headers", "").replace("-image", "")
+        elif package.candidate.source_name in ['linux', 'linux-meta', 'linux-hwe', 'linux-hwe-edge']:
+            source_name = "linux-%s" % package.candidate.version
+        elif package.name.startswith("linux-image") or package.name.startswith("linux-headers") or package.name.startswith("linux-modules") or package.name.startswith("linux-tools"):
+            source_name = "linux-%s" % package.candidate.version
         else:
             source_name = package.candidate.source_name
 
-        # ignore blacklisted packages
+        # ignore packages blacklisted by the user
         for blacklist in self.settings.get_strv("blacklisted-packages"):
             if fnmatch.fnmatch(source_name, blacklist):
                 return
@@ -182,7 +180,7 @@ class APTCheck():
                 update = self.updates[source_name]
                 update.add_package(package)
             else:
-                update = Update(package)
+                update = Update(package, source_name=source_name)
 
                 if source_name in self.named_rules.keys():
                     rule = self.named_rules[source_name]
@@ -195,7 +193,8 @@ class APTCheck():
                             update.level = rule.level
                             break
 
-                self.updates[source_name] = update
+                if update.level < 5:
+                    self.updates[source_name] = update
             if kernel_update:
                 update.type = "kernel"
 
@@ -292,7 +291,7 @@ class APTCheck():
                                                         newline = True
                                                     else:
                                                         if (newline):
-                                                            value = "%s%s" % (value, dline.capitalize())
+                                                            value = "%s%s" % (value, self.capitalize(dline))
                                                         else:
                                                             value = "%s %s" % (value, dline)
                                                         newline = False
@@ -326,12 +325,18 @@ class APTCheck():
                 update.short_description = update.short_description.split("\n")[0]
             if update.short_description.endswith("."):
                 update.short_description = update.short_description[:-1]
-            update.short_description = update.short_description.capitalize()
+            update.short_description = self.capitalize(update.short_description)
             if "& " in update.short_description:
                 update.short_description = update.short_description.replace('&', '&amp;')
             if "& " in update.description:
                 update.description = update.description.replace('&', '&amp;')
 
+    def capitalize(self, string):
+        if len(string) > 1:
+            return (string[0].upper() + string[1:])
+        else:
+            return (string)
+
 if __name__ == "__main__":
     try:
         check = APTCheck()
@@ -342,9 +347,12 @@ if __name__ == "__main__":
         check.apply_aliases()
         check.clean_descriptions()
         check.serialize_updates()
+        if os.path.exists("/usr/bin/pearlinstall-update-pkgcache"):
+            import subprocess
+            subprocess.run(['/usr/bin/pearlinstall-update-pkgcache'], timeout=60)
     except Exception as error:
         print("CHECK_APT_ERROR---EOL---")
         print(sys.exc_info()[0])
-        print(error)
+        print("Error: %s" % error)
         traceback.print_exc()
         sys.exit(1)
