@@ -1,5 +1,5 @@
 #!/usr/bin/python3
-
+import apt
 import subprocess
 import os
 import re
@@ -10,6 +10,8 @@ gi.require_version('Gtk', '3.0')
 gi.require_version('GdkX11', '3.0') # Needed to get xid
 from gi.repository import Gtk, Gdk, GdkX11
 
+from Classes import KERNEL_PKG_NAMES
+
 KERNEL_INFO_DIR = "/usr/share/pearl-kernel-info"
 
 # Using these two as hack to get around the lack of gtk_box_set_center_widget in 3.10
@@ -30,24 +32,21 @@ class InstallKernelThread(threading.Thread):
 
     def run(self):
         cmd = ["pkexec", "/usr/sbin/synaptic", "--hide-main-window",  \
-                "--non-interactive", "--parent-window-id", "%s" % self.window.get_window().get_xid()]
-        cmd.append("-o")
-        cmd.append("Synaptic::closeZvt=true")
-        cmd.append("--progress-str")
-        cmd.append("\"" + _("Please wait, this can take some time") + "\"")
-        cmd.append("--finish-str")
-        if self.remove:
-            cmd.append("\"" + _("The %s kernel was removed") % self.version + "\"")
-        else:
-            cmd.append("\"" + _("The %s kernel was installed") % self.version + "\"")
+                "--non-interactive", "--parent-window-id", "%s" % self.application.window.get_window().get_xid(), "-o", "Synaptic::closeZvt=true"]
         f = tempfile.NamedTemporaryFile()
+        cache = apt.Cache()
+        for name in KERNEL_PKG_NAMES:
+            name = name.replace("VERSION", self.version)
+            if name in cache:
+                pkg = cache[name]
+                if self.remove:
+                    if pkg.is_installed:
+                        pkg_line = "%s\tpurge\n" % name
+                        f.write(pkg_line.encode("utf-8"))
+                else:
+                    pkg_line = "%s\tinstall\n" % name
+                    f.write(pkg_line.encode("utf-8"))
 
-        for pkg in ['linux-headers-%s' % self.version, 'linux-headers-%s-generic' % self.version, 'linux-image-%s-generic' % self.version, 'linux-image-extra-%s-generic' % self.version]:
-            if self.remove:
-                pkg_line = "%s\tpurge\n" % pkg
-            else:
-                pkg_line = "%s\tinstall\n" % pkg
-            f.write(pkg_line.encode("utf-8"))
         cmd.append("--set-selections-file")
         cmd.append("%s" % f.name)
         f.flush()
@@ -202,6 +201,7 @@ class KernelWindow():
         self.application = application
         gladefile = "/usr/share/pearllinux/pearlupdate/kernels.ui"
         builder = Gtk.Builder()
+        builder.set_translation_domain("pearlupdate")
         builder.add_from_file(gladefile)
         self.window = builder.get_object("window1")
         self.window.set_title(_("Kernels"))
@@ -220,14 +220,7 @@ class KernelWindow():
         self.main_stack.add_named(info_box, "info_box")
         builder.get_object("button_continue").connect("clicked", self.on_continue_clicked, main_box)
         builder.get_object("button_help").connect("clicked", self.show_help)
-        builder.get_object("button_continue").set_label(_("Continue"))
         hide_info_checkbox = builder.get_object("checkbutton1").connect("toggled", self.on_info_checkbox_toggled)
-        builder.get_object("checkbutton1").set_label(_("Do not show this message again"))
-
-        builder.get_object("title_warning").set_markup("<span font_weight='bold' size='x-large'>%s</span>" % _("Warning!"))
-        builder.get_object("sub_title_warning").set_markup("<big><b>%s</b></big>" % _("Proceed with caution"))
-        builder.get_object("label_warning").set_markup(_("The Linux kernel is a critical part of the system. Regressions can lead to lack of networking, lack of sound, lack of graphical environment or even the inability to boot the computer. Only install or remove kernels if you are experienced with kernels, drivers, DKMS modules and you know how to recover a non-booting computer."))
-        builder.get_object("label_more_info_1").set_markup("%s" % _("Before installing or removing kernels, please make sure to read the help section, to make sure you know how to select kernels, how to revert updates and how to check your DKMS status."))
 
         # Setup the main kernel page
         stack = Gtk.Stack()
@@ -239,7 +232,6 @@ class KernelWindow():
         kernel_stack_box.pack_start(stack, True, True, 0)
 
         builder.get_object("button_close").connect("clicked", self.hide_window)
-        builder.get_object("button_close").set_label(_("Close"))
 
         kernels = subprocess.check_output("/usr/lib/pearllinux/pearlUpdate/checkKernels.py | sort -r | grep \"###\"", shell = True).decode("utf-8")
         kernels = kernels.split("\n")
@@ -247,11 +239,11 @@ class KernelWindow():
         pages_needed = []
         for kernel in kernels:
             values = kernel.split('###')
-            if len(values) == 9:
+            if len(values) == 7:
                 status = values[0]
                 if status != "KERNEL":
                     continue
-                (status, version_id, version, pkg_version, installed, used, recommended_stability, recommended_security, installable) = values
+                (status, version_id, version, pkg_version, installed, used, installable) = values
                 installed = (installed == "1")
                 used = (used == "1")
                 title = ""
@@ -259,17 +251,6 @@ class KernelWindow():
                     title = _("Active")
                 elif installed:
                     title = _("Installed")
-                recommend = None
-                # if recommended_security == "1":
-                #     recommend = _("Recommended for security")
-                # elif recommended_stability == "1":
-                #     recommend = _("Recommended for stability")
-
-                if recommend is not None:
-                    if title == "":
-                        title = recommend
-                    else:
-                        title = "%s - %s" % (recommend, title)
 
                 installable = (installable == "1")
                 label = version
